/* Agilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis 
 * By Chien-Liang Fok.
 * 
 * Washington University states that Agilla is free software; 
 * you can redistribute it and/or modify it under the terms of 
 * the current version of the GNU Lesser General Public License 
 * as published by the Free Software Foundation.
 * 
 * Agilla is distributed in the hope that it will be useful, but 
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF 
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO 
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER 
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS 
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", 
 * OR OTHER HARMFUL CODE.  
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF 
 * INFORMATION GENERATED USING SOFTWARE. By using Agilla you agree to 
 * indemnify, defend, and hold harmless WU, its employees, officers and 
 * agents from any and all claims, costs, or liabilities, including 
 * attorneys fees and court costs at both the trial and appellate levels 
 * for any loss, damage, or injury caused by your actions or actions of 
 * your officers, servants, agents or third parties acting on behalf or 
 * under authorization from you, as a result of using Agilla. 
 *
 * See the GNU Lesser General Public License for more details, which can 
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */

/*                  tab:4
 *
 *
 * "Copyright (c) 2000-2002 The Regents of the University  of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */
/*                  tab:4
 *  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.  By
 *  downloading, copying, installing or using the software you agree to
 *  this license.  If you do not agree to this license, do not download,
 *  install, copy or use the software.
 *
 *  Intel Open Source License
 *
 *  Copyright (c) 2002 Intel Corporation
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *  Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *      Neither the name of the Intel Corporation nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE INTEL OR ITS
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 */
/* Authors:   Neil Patel
 * History:   created 6/20/2003
 *
 */

/**
 * @author Neil Patel
 * @author Chien-Liang Fok
 */


#ifndef AGILLA_CONSTANTS_H_INCLUDED
#define AGILLA_CONSTANTS_H_INCLUDED

typedef enum {

/* B-class instructions */

  // zero operand instructions
  OPhalt      = 0x00,
  OPloc       = 0x01,
  OPaid       = 0x02,
  OPrand      = 0x03,
  OPcpush     = 0x04,
  OPdepth     = 0x05,
  //OPerr       = 0x06,
  OPvicinity  = 0x06,
  OPclear     = 0x07,
  OPnumnbrs   = 0x08,
  OPrandnbr   = 0x09,
  OPwait      = 0x0a,

  // One operand instructions
  OPinc       = 0x0b,
  OPclearvar  = 0x0c,
  OPinv       = 0x0d,
  OPnot       = 0x0e,
  OPlnot      = 0x0f,
  OPcopy      = 0x10,
  OPpop       = 0x11,
  OPcpull     = 0x12,
  OPsleep     = 0x13,
  OPjumpc     = 0x14,
  OPjumps     = 0x15,
  OPputled    = 0x16,
  OPsmove     = 0x17,
  OPwmove     = 0x18,
  OPsclone    = 0x19,
  OPwclone    = 0x1a,
  OPgetvars   = 0x1b,
  OPsetvars   = 0x1c,
  OPgetnbr    = 0x1d,
  OPcisnbr    = 0x1e,
  OPsense     = 0x1f,
  OPdec       = 0x20,

  // Two and three operand-instructions
  OPdist      = 0x21,
  OPswap      = 0x22,
  OPland      = 0x23,
  OPlor       = 0x24,
  OPand       = 0x25,
  OPor        = 0x26,
  OPshiftr    = 0x27,
  OPshiftl    = 0x28,
  OPadd       = 0x29,
  OPmod       = 0x2a,
  OPceq       = 0x2b, // LSB = 1
  OPcneq      = 0x2c, // LSB = 0
  OPclt       = 0x2d,
  OPcgt       = 0x2e,
  OPclte      = 0x2f,
  OPcgte      = 0x30,
  OPceqtype   = 0x31,
  OPcistype   = 0x32,

  OPout       = 0x33,
  OPinp       = 0x34,
  OPrdp       = 0x35,
  OPin        = 0x36,
  OPrd        = 0x37,
  OPtcount    = 0x38,

  OProut      = 0x39,
  OPrinp      = 0x3a,
  OPrrdp      = 0x3b,
  //OPrinpg     = 0x3c,
  OPrrdpg     = 0x3d,
  OPregrxn    = 0x3e,
  OPderegrxn  = 0x3f,

/*  T class  Instruction format: 0100 ixxx*/
  OPpushrt    = 0x40,
  OPpusht     = 0x48,

/*   E-Class Instruction, format: 0101 ixxx yyyy yyyy yyyy yyyy]*/
  OPpushn     = 0x50,
  OPpushcl    = 0x51,
  OPpushloc   = 0x52,

/*   V-Class Instruction, format:  011i xxxx */
  OPgetvar    = 0x60,  // 5th bit must = 1
  OPsetvar    = 0x70,  // 5th bit must = 0

/*   J class   Instruction format:  10ix xxxx */
  OPrjumpc    = 0x80,  // 100x xxxx conditional relative jump
  OPrjump     = 0xa0,  // 101x xxxx unconditional relative jump

/*   xclass   Instruction format:  11xx xxxx*/
  OPpushc     = 0xc0   // push a constant onto op stack
} AgillaInstruction;

#endif
