/* Agilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis 
 * By Chien-Liang Fok.
 * 
 * Washington University states that Agilla is free software; 
 * you can redistribute it and/or modify it under the terms of 
 * the current version of the GNU Lesser General Public License 
 * as published by the Free Software Foundation.
 * 
 * Agilla is distributed in the hope that it will be useful, but 
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF 
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO 
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO 
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF 
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER 
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS 
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", 
 * OR OTHER HARMFUL CODE.  
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR 
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF 
 * INFORMATION GENERATED USING SOFTWARE. By using Agilla you agree to 
 * indemnify, defend, and hold harmless WU, its employees, officers and 
 * agents from any and all claims, costs, or liabilities, including 
 * attorneys fees and court costs at both the trial and appellate levels 
 * for any loss, damage, or injury caused by your actions or actions of 
 * your officers, servants, agents or third parties acting on behalf or 
 * under authorization from you, as a result of using Agilla. 
 *
 * See the GNU Lesser General Public License for more details, which can 
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */

/**
 * @author Chien-Liang Fok <liang@cse.wustl.edu>
 */
package edu.wustl.mobilab.agilla;
import edu.wustl.mobilab.agilla.messages.*;
import edu.wustl.mobilab.agilla.variables.*;


import java.awt.event.*;
import java.awt.font.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import edu.wustl.mobilab.agilla.rmi.agentInjector.AgentInjectorRMI;
import java.rmi.*;

/**
 *  Injects an agent into the sensor network.
 */
public class Injector extends JFrame implements AgillaConstants {
	private SNInterface sni;
	private AgentSender  agentSender;
	private TupleSpace ts;
	private boolean useRMI;
	private AgentInjectorRMI injectorRMI;
	private String rmihost;
	
    public Injector(String source) throws Exception {
		this.useRMI = source.startsWith("RMI:");
		if (!useRMI) {
			sni = new SNInterface();
			sni.registerListener(new AgillaErrorMsgJ(), new ErrorDisplayer(new AgillaAssembler()));
			agentSender = new AgentSender(sni);
			ts = new TupleSpace(sni);
			//new SimpleTestAgent();     // debug
		}
		else {
			rmihost = source.substring(4);
			System.out.println("rmi host is " + rmihost);
			//let's go get the agent injector
			if (System.getSecurityManager() == null) {
				System.setSecurityManager(new RMISecurityManager());
			}
			try {
				String name="//"+rmihost+"/AgentInjector";
				injectorRMI = (AgentInjectorRMI) Naming.lookup(name);
			} catch (Exception e2) {
				e2.printStackTrace();
			}
			
		}
		
    }
	
	public boolean useRMI() {
		return useRMI;
	}
	
	public String getRMIHost() {
		return rmihost;
	}
	
	public void setMoteIF(MoteIF moteIF) {
		sni.setMoteIF(moteIF);
	}
	
	public void disconnect() {
		sni.disconnect();
	}
	
	public void reset(int address) {
		sendMsg(new AgillaResetMsgJ(address));
	}
	
	public void registerListener(MessageJ msg, MessageListenerJ listener) {
		sni.registerListener(msg, listener);
	}
	
	public void sendMsg(MessageJ msg) {
		try {
			if (sni != null) sni.send(msg);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public void inject(AgillaLocation dest, Agent agent) {
		
		if (useRMI) {
			try {
				injectorRMI.inject(dest, agent);
			} catch(Exception e) {
				e.printStackTrace();
			}
		} else
			agentSender.send(dest, agent);
	}
	
	/**
	 *  This is used to debug the tuple space interaction between
	 * the basestation and the pc.
	 */
	private class SimpleTestAgent implements Runnable, ActionListener {
		
		JFrame frame;
		
		public SimpleTestAgent() {
			frame = new JFrame();
			JButton button = new JButton("Do rdp");
			button.addActionListener(this);
			frame.getContentPane().add(button);
			frame.pack();
			frame.show();
		}
		
		public void actionPerformed(ActionEvent ae) {
//			Tuple template = new Tuple();
//			template.addField(new AgillaTypeVariable(AGILLA_TYPE_NAME));
//			Tuple result = ts.in(template);
//			System.out.println("||----> tuple = " + result);
			new Thread(this).start();
		}
		
		public void run() {
			
//			try {
//				synchronized(this) {
//					wait(2000);
//				}
//			} catch(Exception e) { e.printStackTrace();}
//			System.out.println("||----> SimpleTestAgent running.");
//			Tuple template = new Tuple();
//			template.addField(new AgillaTypeVariable(AGILLA_TYPE_VALUE));
//			template.addField(new AgillaTypeVariable(AGILLA_TYPE_READING, AGILLA_DATA_PHOTO));
//			while(true) {
//				Tuple result = ts.in(template);
//				System.out.println("||----> tuple = " + result);
//			}
			
			Tuple template = new Tuple();
			//template.addField(new AgillaType(AGILLA_TYPE_STRING));
			//template.addField(new AgillaType(AGILLA_TYPE_LOCATION));
			template.addField(new AgillaType(AGILLA_TYPE_STRING));
			template.addField(new AgillaType(AGILLA_TYPE_VALUE));
			System.out.println("||----> Doing an rdp using the following template: \n" + template);
				
			Tuple result = ts.rdp(template);
			System.out.println("||----> Resulting tuple = " + result);
			
		}
	}
}
