// $Id: AgillaTSReqMsgJ.java,v 1.2.2.1 2005/06/01 16:54:03 liang Exp $

/* Agilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Agilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Agilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Agilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * agents from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, agents or third parties acting on behalf or
 * under authorization from you, as a result of using Agilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
package edu.wustl.mobilab.agilla_dsys25.messages;

import edu.wustl.mobilab.agilla_dsys25.*;
import edu.wustl.mobilab.agilla_dsys25.variables.*;

public class AgillaTSReqMsgJ implements MessageJ, AgillaConstants, AgillaOpcodes {
	private AgillaLocation destLoc, replyLoc;
	private short op;
	private Tuple tuple;
	
	public AgillaTSReqMsgJ() {
	}
	
	public AgillaTSReqMsgJ(AgillaLocation destLoc, AgillaLocation replyLoc,
						   short op, Tuple t)
	{
		this.destLoc = destLoc;
		this.replyLoc = replyLoc;
		this.op = op;
		this.tuple = t;
	}
	
	public AgillaTSReqMsgJ(AgillaTSReqMsg msg) {
		this.destLoc = new AgillaLocation(msg.get_destLoc_x(), msg.get_destLoc_y());
		this.replyLoc = new AgillaLocation(msg.get_replyLoc_x(), msg.get_replyLoc_y());
		this.op = msg.get_op();
		
		this.tuple = new Tuple(msg.get_tuple_flags());
		
		short dataIndex = 0;
		short[] tupleData = msg.get_tuple_data();
		int byteIndex = 0;
		for (int i = 0; i < msg.get_tuple_size(); i++) {
			AgillaStackVariable sv = VarUtil.getField(byteIndex, tupleData);
			tuple.addField(sv);
			byteIndex += sv.getSize()+1; // add one for sv type
		}
	}
	
	public int getType() {
		return AM_AGILLATSREQMSG;
	}
	
	public net.tinyos.message.Message toTOSMsg() {
		AgillaTSReqMsg msg = new AgillaTSReqMsg();
		
		msg.set_destLoc_x(destLoc.getx());
		msg.set_destLoc_y(destLoc.gety());
		msg.set_replyLoc_x(replyLoc.getx());
		msg.set_replyLoc_y(replyLoc.gety());
		msg.set_op(op);
		
		msg.set_tuple_flags(tuple.flags());
		msg.set_tuple_size(tuple.getSize());
		
		short dataIndex = 0;
		for (int i = 0; i < tuple.size(); i++) {
			AgillaStackVariable sv = tuple.getField(i);
			short[] bytes = sv.toBytes();
			msg.setElement_tuple_data(dataIndex++, sv.getType());  // save type
			for (int j = 0; j < sv.getSize(); j++) {
				msg.setElement_tuple_data(dataIndex++, bytes[j]);  // save var
			}
		}
		return msg;
	}
	
	private String getString(short op) {
		switch(op) {
			case OProut:
				return "rout";
			case OPrinp:
				return "rinp";
			case OPrrdp:
				return "rrdp";
			default:
				return "UNKNOWN";
		}
	}
	
	public AgillaLocation getReplyLoc() {
		return replyLoc;
	}
	
	public short getOp() {
		return op;
	}
	
	public Tuple getTuple() {
		return tuple;
	}
	
	public String toString() {
		String result = "AGILLA_TS_REQ_MSG:\n";
		result += "\top = 0x" + Integer.toHexString(op) + " ("+getString(op)+")\n";
		result += "\tdestLoc = " + destLoc + "\n";
		result += "\treplyLoc = " + replyLoc + "\n";
		result += "\t"+tuple;
		return result;
	}
	
}

