// $Id: PromiscuousListener.java,v 1.1 2005/05/02 22:53:36 liang Exp $

/* Agilla - A middleware for wireless sensor networks.
 * Copyright (C) 2004, Washington University in Saint Louis
 * By Chien-Liang Fok.
 *
 * Washington University states that Agilla is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Agilla is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT AGILLA IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT AGILLA IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Agilla you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * agents from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, agents or third parties acting on behalf or
 * under authorization from you, as a result of using Agilla.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */
/*									tab:4
 * "Copyright (c) 2000-2003 The Regents of the University  of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 * Copyright (c) 2002-2003 Intel Corporation
 * All rights reserved.
 *
 * This file is distributed under the terms in the attached INTEL-LICENSE
 * file. If you do not find these files, copies can be found by writing to
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300, Berkeley, CA,
 * 94704.  Attention:  Intel License Inquiry.
 */
/* Authors: Chien-Liang Fok <liang@cse.wustl.edu>
 * Date:        May 2 2005
 * Desc:        Main class for promiscuous message listener
 *
 */

/**
 * @author Chien-Liang Fok <liang@cse.wustl.edu>
 */

package edu.wustl.mobilab.agilla;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import edu.wustl.mobilab.agilla.variables.*;
import edu.wustl.mobilab.agilla.messages.*;

public class PromiscuousListener implements AgillaConstants {
	private MoteIF moteIF = null;
	private String source;
	private PhoenixSource psource;
	private SNInterface sni;
	
	public PromiscuousListener(String source, boolean debug) throws Exception {
		this.source = source;
		Debugger.debug = debug;
		connect();
	}
	
	/**
	 * Connects the AgentInjector to the MoteIF.
	 */
	public void connect() {
		try {
			if (source.equals("tossim-serial"))
				moteIF = new MoteIF(PrintStreamMessenger.err);
			else {
				psource = BuildSource.makePhoenix(
					BuildSource.makeArgsSerial(source),
					net.tinyos.util.PrintStreamMessenger.err);
				moteIF = new MoteIF(psource);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		sni = new SNInterface(true);
		sni.setMoteIF(moteIF);
	}
	
	/**
	 * Disconnects the AgentInjector from the MoteIF.
	 */
	public void disconnect() {
		psource.shutdown();
		sni.disconnect();
		moteIF = null;
	}
	
	public boolean isConnected() {
		return moteIF == null;
	}

	public void registerListener(MessageJ msg, MessageListenerJ listener) {
		sni.registerListener(msg, listener);
	}
	
	public void deregisterListener(MessageJ msg, MessageListenerJ listener) {
		sni.deregisterListener(msg, listener);
	}
	
	public static void main(String[] args) {
		try {
			String source = "COM1:mica2"; //"sf@localhost:9001";
			boolean debug = false;
			int index = 0;
			while (index < args.length) {
				String arg = args[index];
				if (arg.equals("-h") || arg.equals("--help")) {
					usage();
					System.exit(0);
				} else if (arg.equals("-comm")) {
					index++;
					source = args[index];
				} else if (arg.equals("-d"))
					debug = true;
				else {
					usage();
					System.exit(1);
				}
				index++;
			}
			new PromiscuousListener(source, debug);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private static void usage() {
		System.err.println("usage: PromiscuousListener[-h|--help|-comm <source>|-d]");
		System.err
			.println("\t-comm <source> where <source> is COMx:[platform] or tossim-serial or RMI:address, default COM4:mica2");
		System.err.println("\t-d for debug mode");
	}
	
}
