/*									tab:4
 *
 *
 * "Copyright (c) 2000-2002 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs and the author appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */
/*									tab:4
 *  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.  By
 *  downloading, copying, installing or using the software you agree to
 *  this license.  If you do not agree to this license, do not download,
 *  install, copy or use the software.
 *
 *  Intel Open Source License 
 *
 *  Copyright (c) 2002 Intel Corporation 
 *  All rights reserved. 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 * 
 *	Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *	Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *      Neither the name of the Intel Corporation nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE INTEL OR ITS
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 */
/**
 * @file FileList.h
 * @author Junaith Ahemed Shahabdeen
 *
 * Linked List definitions and access functions for the
 * Flash File System.
 *
 */
#ifndef _FILE_LIST_H
#define _FILE_LIST_H

typedef struct list_ptr
{
  struct list_ptr *next, *prev;
} list_ptr;

#define INIT_LIST(ptr) do { \
	(ptr)->next = (ptr); (ptr)->prev = (ptr); \
} while (0)

#define get_list_entry(ptr, type, member) \
  ((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))

#define for_each_node_in_list(pos, n, head) \
  for (pos = (head)->next, n = pos->next; pos != (head); pos = n, n = pos->next)

#define for_each_member(pos, head, member)				\
	for (pos = get_list_entry((head)->next, typeof(*pos), member);	\
	     &pos->member != (head); 					\
	     pos = get_list_entry(pos->member.next, typeof(*pos), member))

#define for_each_prev(pos, head) \
  for (pos = (head)->prev; pos != (head); pos = pos->prev)

#define move_list_ptr(pos, head, num, cnt) \
  for (pos = (head)->next;(pos != head) && (cnt < num); pos = pos->next, cnt ++)

static inline void _add_to_list(list_ptr *node,
                               list_ptr *prev,
                               list_ptr *next)
{
  next->prev = node;
  node->next = next;
  node->prev = prev;
  prev->next = node;
}

static inline void _del_node_from_list (list_ptr *prev, list_ptr *next)
{
  next->prev = prev;
  prev->next = next;
}

static inline void add_node(list_ptr *node, list_ptr *head)
{
  _add_to_list (node, head, head->next);
}

static inline void add_node_to_tail(list_ptr *node, list_ptr *head)
{
  _add_to_list (node, head->prev, head);
}

static inline void delete_node(list_ptr *entry)
{
  _del_node_from_list(entry->prev, entry->next);
  entry->next = (void *) 0;
  entry->prev = (void *) 0;
}

static inline int is_list_empty(list_ptr *head)
{
  return head->next == head;
}

static inline void _list_join(list_ptr *list,
                              list_ptr *head)
{
  list_ptr *first = list->next;
  list_ptr *last = list->prev;
  list_ptr *at = head->next;

  first->prev = head;
  head->next = first;
  last->next = at;
  at->prev = last;
}

static inline void join_lists (list_ptr *list, 
                              list_ptr *head)
{
  if (!is_list_empty(list))
    _list_join(list, head->prev);
    //_list_join(list, head);
}

#endif
