
#include "systemUtil.h"
#include "pxa27x_registers_def.h"

#define RESET_DELAY 10000  // delay in 1/3.25 MHz increments = ~3 ms

const char csFatalError[] = "FATAL ERROR: ";
const char csRebootMsg[] = "Rebooting...";

static void configureUART(){
  
  GPIO_SET_ALT_FUNC(46,2,GPIO_IN);
  GPIO_SET_ALT_FUNC(47,1,GPIO_OUT);
  
  //turn on the port's clock
  CKEN |= (CKEN_CKEN5);

  STIER = IER_UUE; //disable all interrupts on the port and enable
  
  STLCR |=LCR_DLAB; //turn on DLAB so we can change the divisor
  STDLL = 8;  //configure to 115200;
  STDLH = 0;
  STLCR &= ~(LCR_DLAB);  //turn off DLAB
    
  STLCR |= 0x3; //configure to 8 bits

  STFCR = FCR_ITL(0) | FCR_TIL | FCR_RESETTF | FCR_TRFIFOE;
  STMCR &= ~MCR_LOOP;
  STMCR |= MCR_OUT2;
}

static void printChar(char data){
  STTHR = data;
  while((STLSR & LSR_TEMT) == 0);
}

static void printString(const char *str,uint32_t strlen){
  uint32_t i;
  
  for(i=0; i<strlen; i++){
    printChar(str[i]);
  }
}

// print one byte in hex
static void printHex(uint8_t num){
  uint32_t i;

  switch(num & 0xF){
    case 0:
      printChar('0');
      break;
    case 1:
      printChar('1');
      break;
    case 2:
      printChar('2');
      break;
    case 3:
      printChar('3');
      break;
    case 4:
      printChar('4');
      break;
    case 5:
      printChar('5');
      break;
    case 6:
      printChar('6');
      break;
    case 7:
      printChar('7');
      break;
    case 8:
      printChar('8');
      break;
    case 9:
      printChar('9');
      break;
    case 10:
      printChar('A');
      break;
    case 11:
      printChar('B');
      break;
    case 12:
      printChar('C');
      break;
    case 13:
      printChar('D');
      break;
    case 14:
      printChar('E');
      break;
    case 15:
      printChar('F');
      break;
  }
}


static void printHex32(uint32_t num){
  int i;
  
  printChar('0');
  printChar('x');
  
  for(i=7; i>=0; i--){
    printHex(num>>(4*i));
  }
}

static void printDecimal(uint32_t num){
  int i,numDigits =1;
  char buf[10];

  for(i=0;i<10;i++){
    buf[9-i] = '0' + (num % 10);
    if(buf[9-i] != '0'){
      numDigits = i+1;
    }
    num = num/10;
  }
  printString((const char *)(buf+10-numDigits),numDigits);
}

void printAssertMsg(const char* file, uint32_t line, char *condition){
  
  configureUART();
  
  printString(csFatalError, strlen(csFatalError));
  printString(file,strlen(file));
  printChar(':');
  printDecimal(line);
  printString(": Assertion failed: ",strlen(": Assertion failed: "));
  printString(condition, strlen(condition));
  printChar('\r');
  printChar('\n');

  resetNode();
}

void printFatalErrorMsg(const char *msg, uint32_t status){
  uint32_t i, len;
  
  configureUART();
  
  printString(csFatalError, strlen(csFatalError)); 
  printString(msg, strlen(msg));
  printDecimal(status);
  printChar('\r');
  printChar('\n');
  
  resetNode();
}

void resetNode(){
  
  configureUART();
  
  printString(csRebootMsg, strlen(csRebootMsg));
  printChar('\r');
  printChar('\n');
  
  OSMR3 = OSCR0 + RESET_DELAY;
  OWER = 1;
  while(1);
}
